#include "plane.h"
#include "material.h"
#include "RTConfig.h"

// Pour ne pas avoir a mettre qglviewer:: devant les classes
// definies dans le namespace qglviewer.
using namespace qglviewer;

Plane::Plane():bradius_(1.0f) {
}

void Plane::draw() const
{
	if(!RTConfig::getInstance()->drawPlane)
		return;

	//Disable rendering for gpurt safe mode
	/*if(RTConfig::getInstance()->gpurtSafeMode)
		return;*/

  glMatrixMode(GL_MODELVIEW);
  glPushMatrix();

  glPushAttrib(GL_ALL_ATTRIB_BITS);

  glMultMatrixd(frame().matrix());
  glColor3fv(material().diffuseColor());

  float colorMat[] = { material().diffuseColor().r, material().diffuseColor().g, material().diffuseColor().b, 1.0f };
  glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, colorMat);
  float colorMatSpec[] = { material().specularColor().r, material().specularColor().g, material().specularColor().b, 1.0f };
  glMaterialfv(GL_FRONT, GL_SPECULAR, colorMatSpec);

  glMateriali(GL_FRONT, GL_SHININESS, (int)material().specularCoefficient());

  glBegin(GL_QUADS);
	glVertex3f(-1000,-1000,0);
	glVertex3f(1000,-1000,0);
	glVertex3f(1000,1000,0);
	glVertex3f(-1000,1000,0);
  glEnd();
  glPopAttrib();
  glPopMatrix();
}

void Plane::initFromDOMElement(const QDomElement& e)
{
  Object::initFromDOMElement(e);
 /* if(e.hasAttribute("radius"))
    this->setRadius(e.attribute("radius", "0").toFloat());*/

  //Plane equation computation
  Vec center=this->frame().position();
  Vec normal=this->frame().inverseTransformOf(Vec(0,0,1));
  float D=-normal.x*center.x-normal.y*center.y-normal.z*center.z;

  equation_[0]=normal.x;
  equation_[1]=normal.y;
  equation_[2]=normal.z;
  equation_[3]=D;
}

bool Plane::intersect(const Ray& ray, Hit& hit) const
{
  float t;

  //Vec center=this->frame().transformOf(Vec(0,0,0));
  //Vec normal=this->frame().transformOf(Vec(0,0,1));

  Vec center=this->frame().position();
  Vec normal=this->frame().inverseTransformOf(Vec(0,0,1));


  float D=-normal.x*center.x-normal.y*center.y-normal.z*center.z;

  t = -(D + normal*ray.start() ) / (normal*ray.direction());

  float tmin=RTConfig::getInstance()->getRayTMin();// 0.0001;

  if(hit.time()>t && t>tmin){
	  hit.setTime(t);
	  hit.setIntersection( ray.direction()*t+ray.start() );
	  hit.setMaterial(this->material());
	  hit.setNormal(normal);
	  hit.setTangentSpace(this->frame().inverseTransformOf(Vec(1,0,0)), this->frame().inverseTransformOf(Vec(0,1,0)));

	  // "Partie XI: Textures"
	  Frame invFrame=this->frame().inverse();
	  Vec inter2D=this->frame().transformOf(hit.intersection());
	  hit.setUV(inter2D.x, inter2D.y );

	  return true;
  }

  return false;
}
