#include "sphere.h"
#include "material.h"
#include "RTConfig.h"

// Pour ne pas avoir a mettre qglviewer:: devant les classes
// definies dans le namespace qglviewer.
using namespace qglviewer;

Sphere::Sphere():slices(100), stacks(100) {
  setRadius(1.0);
}

void Sphere::draw() const
{
  glMatrixMode(GL_MODELVIEW);
  glPushMatrix();

  glPushAttrib(GL_ALL_ATTRIB_BITS);

  glMultMatrixd(frame().matrix());
  glColor3fv(material().diffuseColor());


//???????????????????????????
//->Ca c'est pour l'eclairage OpenGL, pour faire du blinn-phong :)
  float colorMat[] = { material().diffuseColor().r, material().diffuseColor().g, material().diffuseColor().b, 1.0f };
  glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, colorMat);
  float colorMatSpec[] = { material().specularColor().r, material().specularColor().g, material().specularColor().b, 1.0f };
  glMaterialfv(GL_FRONT, GL_SPECULAR, colorMatSpec);
  glMateriali(GL_FRONT, GL_SHININESS, (int)material().specularCoefficient());
//???????????????????????????

  GLUquadricObj *quad=gluNewQuadric();
  gluSphere(quad,this->radius() , slices, stacks);

  glPopAttrib();
  glPopMatrix();
}

// Build a sphere object
void Sphere::initFromDOMElement(const QDomElement& e)
{
  Object::initFromDOMElement(e);
  if(e.hasAttribute("radius"))
    this->setRadius(e.attribute("radius", "0").toFloat());
}


bool Sphere::intersect(const Ray& ray, Hit& hit) const
{
  // In the frame of the sphere (center = (0,0,0) radius = r), 
  // with Ro the origin of the ray, and Rd its direction,
  // we have to solve the equation: Rd*Rd*t + 2*Rd*Ro*t + Ro*Ro - r = 0
  //                                   a *t +    b   *t +      c     = 0
  // BUT here, we are in the world frame. We must replace us in the frmae of the sphere
  // SO:
  //    Rd = ray.direction()  --> unchanged  
  //    Ro = ray.start()-this->frame().position()

  float a=1;  // Rd is normalized
  float b=2.0f*(ray.direction())*(ray.start()-this->frame().position());
  float c=(ray.start()-this->frame().position())*(ray.start()-this->frame().position())-this->radius()*this->radius();

  // Compute the discriminant
  float d=(b*b-4.0f*a*c);
  float t;

  // If d<0, no solution
  if(d<0){
	  return false;
  }

  // To avoid noise if the ray intersects the object itself;
  float tmin=RTConfig::getInstance()->getRayTMin();// 0.0001;

  // If d==0, one solution (ie the ray intersects "on" the tangent of the sphere: nearly impossible with float precision!)
  if(d==0)
	t=-b/(2*a);
  
  // If d>0, 2 solutions (front and back of the sphere)
  else{
	float t0=(-b-sqrt(d))/(2.0f*a);
	float t1=(-b+sqrt(d))/(2.0f*a);

	// Gets the minimum t (ie the first intersection from the origin of the ray and the object)
	//t=(t0<t1)?t0:t1;

	if( t0<tmin && t1>tmin)
		t=t1;
	else if(t1<tmin && t0>tmin)
		t=t0;
	else
		t=(t0<t1)?t0:t1;

  }

  // If the "new computed t" is < to the hit.time, we keep it (this intersection is the nearest)
  // (we must have t>tmin in order not to be behind the origin of the ray)
  if(hit.time()>t && t>tmin){
	  // Modify all the parameters of the hit
	  hit.setTime(t);
	  hit.setIntersection( ray.direction()*t+ray.start() );
	  hit.setMaterial(this->material());

	  // Compute the normale at the intersection (ie direction of the radius from the center of the sphere)
	  Vec normale = hit.intersection()-this->frame().position();
	  normale.normalize();
	  hit.setNormal(normale);
	  
	  // "Partie XI: Textures"
	  qglviewer::Vec pos = hit.intersection() - this->frame().position();
	  hit.setUV((atan2(pos.y,pos.x)+M_PI)/(1.0f*M_PI), (acosf(pos.z/this->radius()))/(M_PI) );

	  // Extension: Bump Mapping
	  Vec perturbNormale = hit.material().bumpMapColor(hit.u(), hit.v());
	  Vec rotatePerturbNormale;
	
	  if ( (perturbNormale.x == -1) && (perturbNormale.y == -1) && (perturbNormale.z == -1) ) 
		  rotatePerturbNormale = hit.normal();
	  else {
		  Vec rotatePerturbNormaleTmp;
		  float u = hit.u() * (2.0f*M_PI) + M_PI;
		  float v = hit.v() * M_PI;
		  // Rotation along z-axis
		  rotatePerturbNormaleTmp.x = cos(u) * perturbNormale.x - sin(u) * perturbNormale.y;
		  rotatePerturbNormaleTmp.y = sin(u) * perturbNormale.x + cos(u) * perturbNormale.y;
		  rotatePerturbNormaleTmp.z = perturbNormale.z;

		  // Rotation along x-axis
		  rotatePerturbNormale.x = rotatePerturbNormaleTmp.x;
		  rotatePerturbNormale.y = cos(v) * rotatePerturbNormaleTmp.y - sin(v) * rotatePerturbNormaleTmp.z;
		  rotatePerturbNormale.z = sin(v) * rotatePerturbNormaleTmp.y + cos(v) * rotatePerturbNormaleTmp.z;
	  }

	  Vec newNormale = /*hit.normal() +*/ rotatePerturbNormale;
	  newNormale.normalize();
	  hit.setbumpMapNormal(newNormale);

	  hit.setCoeffSpecularMap(hit.material().specularMapColor(hit.u(), hit.v()));




	  return true;
  }

  return false;
}
